from flask import Blueprint, render_template, request, redirect, url_for, flash, abort
from ..nosql import get_nosql
from ..extensions import caching, caching_unless

import re

bp = Blueprint('api', __name__, url_prefix='/api')

@bp.route('/websub/<cap>', methods=['GET', 'POST'])
def websub(cap):
    if request.method == 'GET':
        topic = request.args.get('hub.topic')
        challenge = request.args.get('hub.challenge')
        mode = request.args.get('hub.mode')
        lease_seconds = request.args.get('hub.lease_seconds')
        
        if mode not in ['subscribe', 'unsubscribe']:
            return abort(400)
        
        if not get_nosql().websub_existsCallback(cap):
            return abort(404)

        if mode == 'unsubscribe':
            get_nosql().websub_retireCallback(cap)
            return challenge
        
        if not all([topic, challenge, mode, lease_seconds]):
            return abort(400)
        
        if not get_nosql().websub_activateCallback(cap, lease_seconds):
            return abort(500)
            
        return challenge
    
    if get_nosql().websub_existsCallback(cap):
        if not get_nosql().websub_savePost(cap, request.data):
            return abort(500)
        return '', 202
    
    return abort(404)
    
@bp.route('/queue/<cap>', methods=['POST'])
def queue(cap):
    # if endpoint does not exist
    if not get_nosql().queue_isActive(cap):
        return abort(404)
    
    videoId = request.form.get('v')
    
    # if request is not valid
    if not videoId:
        return abort(400)
        
    # if requested string is not correct
    if not re.match(r"^[a-zA-Z0-9_-]{11}$", videoId):
        return abort(422)
        
    # if given string is already in the archive
    if get_nosql().check_exists(videoId):
        return abort(409)
    
    # try to insert
    if get_nosql().queue_insertQueue(videoId, cap):
        return '', 202
    else:
        return abort(409)