from flask import Blueprint, redirect, url_for, render_template, request, session, flash, current_app, redirect
from ..extensions import limiter, caching, caching_unless, oidc
from ..nosql import get_nosql

from time import sleep

bp = Blueprint('auth', __name__, url_prefix='/auth')

@bp.route('')
@caching.cached()
def base():
    return redirect(url_for('auth.login'))
    
@bp.route('/logout')
def logout():
    session.pop('username', None)
    flash('You have been logged out')
    return redirect(url_for('auth.login'))
    
@bp.route('/login', methods=['GET', 'POST'])
@limiter.limit('10 per day', override_defaults=False)
@caching.cached(unless=caching_unless)
def login():
    if request.method == 'POST':
        password = request.form.get('password', None)
        
        if current_app.config.get('DEBUG'):
            session['username'] = 'DEBUG ADMIN'
            flash('You have been logged in')
            return redirect(request.args.get('next', url_for('admin.base')))
        
        if not password:
            flash('Password was empty')
            return redirect(url_for('auth.login'))
        
        sleep(0.3)
        flash('Wrong password')
        return redirect(url_for('auth.login'))
        
    return render_template('login.html')

@bp.route('/oidc', methods=['GET'])
def start_oidc():
    return redirect(oidc.generate_redirect(), code=302)

@bp.route('/callback', methods=['POST'])
@limiter.limit('30 per day', override_defaults=False)
@caching.cached(unless=caching_unless)
def callback():
    state = request.form.get('state', None)
    id_token = request.form.get('id_token', None)
    
    if request.form.get('error', None):
        return f'We got an error from the authentication provider with the message: {request.form.get("error_description", None)}', 400
    
    if state is None or id_token is None:
        return 'Request error', 400
    
    if not oidc.state_check(state):
        return 'CSRF Error, state is not valid', 400

    sub = oidc.check_bearer(id_token)
    
    if not sub:
        return f'Invalid JWT token we got: {id_token}', 400
    
    if not get_nosql().get_user(sub):
        return f'Authentication successful, but you are not allowed to access authenticated pages. Please report this ID to the administrators if you want access: {sub}', 403
    
    session['username'] = sub
    flash('You have been logged in')
    return redirect(request.args.get('next', url_for('admin.base')))