from flask import Blueprint, render_template, flash, url_for, redirect
from ..nosql import get_nosql
from ..extensions import caching, caching_unless

bp = Blueprint('channel', __name__, url_prefix='/channel')

@bp.route('')
@caching.cached(unless=caching_unless)
def base():
    channels = []
    channelIds = get_nosql().list_all_channels()
    
    for channelId in channelIds:
        channel = get_nosql().get_channel_info(channelId)
        channel['video_count'] = get_nosql().get_channel_videos_count(channelId)
        channels.append(channel)
        
    channels = sorted(channels, key=lambda x: x.get('added_date'), reverse=True)
        
    return render_template('channel/index.html', channels=channels)
    
@bp.route('/<channelId>')
@caching.cached(unless=caching_unless)
def channel(channelId):
    channelInfo = get_nosql().get_channel_info(channelId)

    if not channelInfo:
        flash('That channel ID does not exist in the system')
        return redirect(url_for('channel.base'))
        
    videoIds = get_nosql().get_channel_videoIds(channelId)
    
    videos = []
    for videoId in videoIds:
        videos.append(get_nosql().get_video_info(videoId, limited=True))
        
    videos = sorted(videos, key=lambda x: x.get('upload_date', '19700101'), reverse=True)

    return render_template('channel/channel.html', channel=channelInfo, videos=videos)
    
@bp.route('/orphaned')
@caching.cached(unless=caching_unless)
def orphaned():
    videoIds = get_nosql().get_orphaned_videos()
    
    videos = []
    for videoId in videoIds:
        videos.append(get_nosql().get_video_info(videoId, limited=True))
        
    videos = sorted(videos, key=lambda x: x.get('epoch', 0), reverse=True)

    return render_template('channel/orphaned.html', videos=videos)
    
@bp.route('/recent')
@caching.cached(unless=caching_unless)
def recent():
    videoIds = get_nosql().get_recent_videos()
    
    videos = []
    for videoId in videoIds:
        videos.append(get_nosql().get_video_info(videoId, limited=True))
        
    videos = sorted(videos, key=lambda x: x.get('epoch', 0), reverse=True)

    return render_template('channel/recent.html', videos=videos)