from flask import Blueprint, render_template, request, flash, redirect, url_for
from ..nosql import get_nosql
from ..extensions import limiter, caching, caching_unless

bp = Blueprint('search', __name__, url_prefix='/search')

@bp.route('', methods=['GET', 'POST'])
@limiter.limit('50 per day', override_defaults=False)
@caching.cached(unless=caching_unless)
def base():
    if request.method == 'POST':
        task = request.form.get('task')
        
        if task == 'search':
            query = request.form.get('query')
            
            if not 3 <= len(query) <= 64:
                flash('Query too short or too long, must be between 3 and 64')
                return redirect(url_for('search.base'))
                
            results = get_nosql().search_videos(query)
            
            return render_template('search/index.html', results=results, query=query)
                
        
    return render_template('search/index.html', stats=get_nosql().gen_stats())