from flask_limiter import Limiter
from flask_limiter.util import get_remote_address

from flask_caching import Cache

from celery import Celery, Task

from .oidc import OIDC

from flask import Flask, request, session

def celery_init_app(app: Flask) -> Celery:
    class FlaskTask(Task):
        def __call__(self, *args: object, **kwargs: object) -> object:
            with app.app_context():
                return self.run(*args, **kwargs)

    celery_app = Celery(app.name, task_cls=FlaskTask)
    celery_app.config_from_object(app.config["CELERY"])
    celery_app.set_default()
    app.extensions["celery"] = celery_app
    return celery_app

def caching_unless(*args, **kwargs):
    # if it is not a get request
    if request.method != 'GET':
        return True
    
    # if username is defined in session cookie
    if session.get('username'):
        return True
    
    # in the case that a user is not logged in but a message needs to be flashed, do not cache page
    if session.get('_flashes'):
            return True
        
    # do cache page
    return False
    
def caching_v_parameter(*args, **kwargs):
    return request.args.get('v')

limiter = Limiter(
        get_remote_address,
        default_limits=['86400 per day', '3600 per hour'],
        storage_uri="memory://",
    )
    
caching = Cache()

oidc = OIDC()